/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RgbViewer.h"

#include <InstrumentManager/InstrumentManager.h>
#include <ics/Utilities/Calculations.h>
#include <boost/lexical_cast.hpp>
#include "controllers/common/family/Families.h"

#include <hdf5/serial/hdf5.h>
#include <hdf5/serial/hdf5_hl.h>

#include <fstream>
#include <iostream>

namespace viewer {

using namespace std;
using namespace common;
using namespace boost;
const std::string RgbViewer::TYPE = "rgbviewer";

RgbViewer::RgbViewer(const std::string& name) :
		utilities::ImageData(name), controller::Stoppable(this) {
	setFamily(family::TOOLS, family::TOOLS);
	LiveData.init(this, NOSAVE, "LiveData");
	RGBData.init(this, NOSAVE, "RgbData");
	height.init(this, SAVE, "height");
	width.init(this, SAVE, "width");
	pixelFormat.init(this, SAVE, "type");

	m_CamDrivers.init(this, "camDrivers");
	registerRefresher(LiveData, &RgbViewer::refreshModeProperty, this);

}

RgbViewer::~RgbViewer() {

}

void RgbViewer::postConfiguration() {
	registerUpdater(m_CamDrivers->RGBData, &RgbViewer::updateRGBData, this);
	height = m_CamDrivers->height();
	width = m_CamDrivers->width();
	pixelFormat = m_CamDrivers->pixelFormat();

//	initialized = initApplication();
}

/*
 * refreshModeProperty
 */
void RgbViewer::refreshModeProperty(bool value) throw (CannotSetValue) {

}

bool RgbViewer::initApplication() {

	return true;
}

void RgbViewer::start() {

	this->type = m_CamDrivers->pixelFormat();
	do {
		m_CamDrivers->execute(START_COMMAND);

//		setContentFromBinaryData(m_CamDrivers->RGBData());
//	cout<< "started"<< m_CamDrivers->RGBData().size()<<endl;
	} while ((LiveData() == true));

	hid_t file_id;
	unsigned char* buf2 = new unsigned char[m_CamDrivers->RGBData().size() + 1];
	memcpy(buf2, m_CamDrivers->RGBData().c_str(), m_CamDrivers->RGBData().size());
	buf2[m_CamDrivers->RGBData().size()] = '\0';

	/* create a new HDF5 file using default properties. */
	file_id = H5Fcreate("ex_image1.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

	/* make dataset */
	H5IMmake_image_24bit(file_id, "image1", (hsize_t) m_CamDrivers->width(), (hsize_t) m_CamDrivers->height(),
			"INTERLACE_PIXEL", buf2);

	delete buf2;

	/* close the file. */
	H5Fclose(file_id);

}
void RgbViewer::stop() {

	LiveData = false;
}

void RgbViewer::subscriberLoop() {

}

void RgbViewer::updateRGBData() {
	RGBData = m_CamDrivers->RGBData();
	this->type = m_CamDrivers->pixelFormat();

	setContentFromBinaryData(RGBData());
}

}

