/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef RGBVIEWER_H
#define RGBVIEWER_H

#include "controllers/common/utilities/imageviewer/ImageData.h"
#include "controllers/common/acquisition/ImageGeneratable.h"
#include "drivers/imagingsources/dfk31au03/Dfk31Au03Driver.h"

#define WIDTH         1024
#define HEIGHT        768

namespace viewer {

class RgbViewer: public utilities::ImageData,
		public acquisition::ImageGeneratable,
		public controller::Stoppable {

public:
	//! Type of controller
	static const std::string TYPE;

	RgbViewer(const std::string& name);
	virtual ~RgbViewer();

	virtual void postConfiguration();
	virtual void refreshModeProperty(bool value) throw (CannotSetValue);

	void updateNumor();
	virtual void start();
	virtual void stop();

	Property<std::string> serverEndpoint;
	Property<bool> initialized;
	Property<bool> test;
	Property<int32> testNumor;
	Property<bool> LiveData;
	Property<string> RGBData;
	Property<int32> height;
	Property<int32> width;
	Property<string> pixelFormat;

	DriverPtr<dfk41au03::Dfk31Au03Driver> m_CamDrivers;

private:
	bool initApplication();
	void sendNumor(int32 numor, bool live);
	void subscriberLoop();

	void updateRGBData();

};

}

#endif
