/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealDfk31Au03Driver.h"
#include "Dfk31Au03Driver.h"

#include <libv4l2.h>
#include <usb.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <linux/videodev2.h>
namespace dfk41au03 {

char* RealDfk31Au03Driver::DEV_NAME = "/dev/video1";

void RealDfk31Au03Driver::v4lconvert_yuyv_to_rgb24(const unsigned char *src, unsigned char *dest, int width, int height,
		int stride) {
	int j = 0;
	int y;

	for (y = 0; y < stride - 4; y = y + 4) {

		int u = src[y + 1];

		int v = src[y + 3];
		//            printf("src[3] %d\n",src[3]);

		int u1 = (((u - 128) << 7) + (u - 128)) >> 6;
		int rg = (((u - 128) << 1) + (u - 128) + ((v - 128) << 2) + ((v - 128) << 1)) >> 3;
		int v1 = (((v - 128) << 1) + (v - 128)) >> 1;

		dest[j + 0] = CLIP(src[y + 0] + v1);
		dest[j + 1] = CLIP(src[y + 0] - rg);
		dest[j + 2] = CLIP(src[y + 0] + u1);

		dest[j + 3] = CLIP(src[y + 2] + v1);
		dest[j + 4] = CLIP(src[y + 2] - rg);
		dest[j + 5] = CLIP(src[y + 2] + u1);
		j = j + 6;

		// printf("convert size : %d\n",j);
		// src += 4;

		//  src += stride - (width * 2);
	}
}

void RealDfk31Au03Driver::errno_exit(const char *s) {
	fprintf(stderr, "%s error %d, %s\n", s, errno, strerror(errno));

}

int RealDfk31Au03Driver::xioctl(int fd, int request, void *arg) {
	int r;
	do {
		r = ioctl(fd, request, arg);
	} while (-1 == r && EINTR == errno);
	return r;
}

void RealDfk31Au03Driver::process_image(const void *p_buf, const int len_buf) {
	FILE *fp;
	int i;

	v4lconvert_yuyv_to_rgb24((unsigned char*) p_buf, (unsigned char*) dest_buf, 1024, 768, len_buf);

//	  fp = fopen(OUTFILE_NAME,"wb");
//	  if(fp == NULL) return;
//	  fprintf(fp, "P6\n%d %d 255\n",
//	                         1024, 768);
//	  fwrite(dest_buf,2359296,1,fp);
//	  fclose(fp);
	std::string localstring(reinterpret_cast<const char *>(dest_buf), 1024 * 768 * 3);
	owner()->RGBData = localstring;		//((char *)buffers[buf.index].start);

	//printf("Saved\n");
}

int RealDfk31Au03Driver::read_frame(int count) {
	struct v4l2_buffer buf;

	CLEAR(buf);
	buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	buf.memory = V4L2_MEMORY_MMAP;
	if (-1 == xioctl(fd, VIDIOC_DQBUF, &buf)) {
		switch (errno) {
		case EAGAIN:
			return 0;
		case EIO:
			/* Could ignore EIO, see spec. */
			/* fall through */
		default:
			errno_exit("VIDIOC_DQBUF");
		}
	}
	assert(buf.index < n_buffers);
	if (count == 0) {
		process_image(buffers[buf.index].start, buffers[buf.index].length);
	}
	if (-1 == xioctl(fd, VIDIOC_QBUF, &buf)) {
		errno_exit("VIDIOC_QBUF");
	}

	return 1;
}

void RealDfk31Au03Driver::mainloop() {
	uint32 count;

	count = COUNT_IGNORE;

	while (count-- > 0) {
		for (;;) {
			fd_set fds;
			struct timeval tv;
			int r;
			FD_ZERO(&fds);
			FD_SET(fd, &fds);
			/* Timeout. */
			tv.tv_sec = 2;
			tv.tv_usec = 0;
			r = select(fd + 1, &fds, NULL, NULL, &tv);
			if (-1 == r) {
				if (EINTR == errno)
					continue;
				errno_exit("select");
			}
			if (0 == r) {
				fprintf(stderr, "select timeout\n");
			}
			if (read_frame(count))
				break;
			/* EAGAIN - continue select loop. */
		}

	}

}
void RealDfk31Au03Driver::stop_capturing() {
	enum v4l2_buf_type type;

	type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	if (-1 == xioctl(fd, VIDIOC_STREAMOFF, &type))
		errno_exit("VIDIOC_STREAMOFF");
}

void RealDfk31Au03Driver::start_capturing(void) {
	uint32 i;
	enum v4l2_buf_type type;

	for (i = 0; i < n_buffers; ++i) {
		struct v4l2_buffer buf;
		CLEAR(buf);
		buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		buf.memory = V4L2_MEMORY_MMAP;
		buf.index = i;

		if (-1 == xioctl(fd, VIDIOC_QBUF, &buf))
			errno_exit("VIDIOC_QBUF");
	}
	type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	if (-1 == xioctl(fd, VIDIOC_STREAMON, &type))
		errno_exit("VIDIOC_STREAMON");
}

void RealDfk31Au03Driver::uninit_device() {
	uint32 i;

	for (i = 0; i < n_buffers; ++i)
		if (-1 == munmap(buffers[i].start, buffers[i].length))
			errno_exit("munmap");
	free(buffers);
}

void RealDfk31Au03Driver::init_mmap() {
	struct v4l2_requestbuffers req;

	CLEAR(req);

	req.count = 2;
	req.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	req.memory = V4L2_MEMORY_MMAP;

	if (-1 == xioctl(fd, VIDIOC_REQBUFS, &req)) {
		if (EINVAL == errno) {
			fprintf(stderr, "%s does not support "
					"memory mapping\n", DEV_NAME);
		} else {
			errno_exit("VIDIOC_REQBUFS");
		}
	}
	if (req.count < 2) {
		fprintf(stderr, "Insufficient buffer memory on %s\n", DEV_NAME);
	}
	buffers = (struct buffer*) calloc(req.count, sizeof(*buffers));

	if (!buffers) {
		fprintf(stderr, "Out of memory\n");
	}

	for (n_buffers = 0; n_buffers < req.count; ++n_buffers) {
		struct v4l2_buffer buf;
		CLEAR(buf);
		buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		buf.memory = V4L2_MEMORY_MMAP;
		buf.index = n_buffers;

		if (-1 == xioctl(fd, VIDIOC_QUERYBUF, &buf))
			errno_exit("VIDIOC_QUERYBUF");

		// printf("Buffer length : %d : %d\n",n_buffers,buf.length);

		buffers[n_buffers].length = buf.length;
		buffers[n_buffers].start = mmap(NULL /* start anywhere */, buf.length,
		PROT_READ | PROT_WRITE /* required */,
		MAP_SHARED /* recommended */, fd, buf.m.offset);

		if (MAP_FAILED == buffers[n_buffers].start)
			errno_exit("mmap");
	}
}

void RealDfk31Au03Driver::init_device() {
	struct v4l2_capability cap;
	struct v4l2_cropcap cropcap;
	struct v4l2_crop crop;
	struct v4l2_format fmt;
	uint32 min;

	if (-1 == xioctl(fd, VIDIOC_QUERYCAP, &cap)) {
		if (EINVAL == errno) {
			fprintf(stderr, "%s is no V4L2 device\n", DEV_NAME);
		} else {
			errno_exit("VIDIOC_QUERYCAP");
		}
	}
//	std::string v(reinterpret_cast<const char *>(dest_buf),  2359296);

	stringstream myStreamString;
	myStreamString << cap.card;
	owner()->version = myStreamString.str();

	if (!(cap.capabilities & V4L2_CAP_VIDEO_CAPTURE)) {
		fprintf(stderr, "%s is no video capture device\n", DEV_NAME);
	}
	if (!(cap.capabilities & V4L2_CAP_STREAMING)) {
		fprintf(stderr, "%s does not support streaming i/o\n", DEV_NAME);
	}
	/* Select video input, video standard and tune here. */
	CLEAR(cropcap);

	cropcap.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;

	if (0 == xioctl(fd, VIDIOC_CROPCAP, &cropcap)) {
		crop.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		crop.c = cropcap.defrect; /* reset to default */

		if (-1 == xioctl(fd, VIDIOC_S_CROP, &crop)) {
			switch (errno) {
			case EINVAL:
				/* Cropping not supported. */
				break;
			default:
				/* Errors ignored. */
				break;
			}
		}
	} else {
		/* Errors ignored. */
	}
	CLEAR(fmt);

	fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	fmt.fmt.pix.width = IMAGE_WIDTH;
	fmt.fmt.pix.height = IMAGE_HEIGHT;
	fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUYV;
	fmt.fmt.pix.field = V4L2_FIELD_INTERLACED;

	if (-1 == xioctl(fd, VIDIOC_S_FMT, &fmt))
		errno_exit("VIDIOC_S_FMT");

	/* Note VIDIOC_S_FMT may change width and height. */

	struct v4l2_streamparm fps;
	memset(&fps, 0, sizeof(fps));
	fps.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	if (ioctl(fd, VIDIOC_G_PARM, &fps) < 0) {
		printf("Couldn't query v4l fps!\n");
	}
	owner()->height = fmt.fmt.pix.height;
	owner()->width = fmt.fmt.pix.width;
	//owner()->height=fmt.fmt.pix.pixelformat ;
	owner()->bitPerPixel = RGB24BIT;
	owner()->pixelFormat = RGB;

	fps.parm.capture.readbuffers = 0;
	fps.parm.capture.timeperframe.numerator = 2;
	fps.parm.capture.timeperframe.denominator = 15;
	if (ioctl(fd, VIDIOC_S_PARM, &fps) < 0) {
		printf("Couldn't set v4l fps!\n");
	}

	init_mmap();
}

void RealDfk31Au03Driver::close_device() {
	if (-1 == close(fd))
		errno_exit("close");
	fd = -1;
}

void RealDfk31Au03Driver::open_device() {

	char *device = new char[owner()->video().length() + 1]; // or
	// char y[100];

	std::strcpy(device, owner()->video().c_str());

	fd = open(device, O_RDWR /* required */| O_NONBLOCK, 0);
	delete[] device;
	if (-1 == fd) {
		fprintf(stderr, "Cannot open '%s': %d, %s\n", device, errno, strerror(errno));

	}

}

/*
 * Constructor
 */
RealDfk31Au03Driver::RealDfk31Au03Driver(Dfk31Au03Driver* owner) :
		Dfk31Au03State(owner) {
	/* Empty */
}

/*
 * Destructor
 */
RealDfk31Au03Driver::~RealDfk31Au03Driver() {
	uninit_device();
	close_device();

}

/*
 * init
 */
void RealDfk31Au03Driver::init() {

	open_device();
	init_device();

	readInfos();
	readStatus();
}

/*
 * clear
 */
void RealDfk31Au03Driver::clear() {

}

/*
 * writeParam
 */
void RealDfk31Au03Driver::writeParam() {

}

/*
 * synchroniseRead
 */
void RealDfk31Au03Driver::synchroniseRead() {

	start_capturing();
	mainloop();
	stop_capturing();
	/*
	 * Calculate the detector sum here
	 */

	owner()->m_SynchroniseReadTerminated = true;
}

/*
 * read
 */
void RealDfk31Au03Driver::read() {
}

/*
 * start
 */
void RealDfk31Au03Driver::start() {

	synchroniseRead();

	stop();
}

/*
 * resume
 */
void RealDfk31Au03Driver::resume() {
}

/*
 * pause
 */
void RealDfk31Au03Driver::pause() {
}

/*
 * stop
 */
void RealDfk31Au03Driver::stop() {
	if (owner()->startActivated() == true) {
		owner()->startActivated = false;
		owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	}

}

/*
 * readStatus
 */
void RealDfk31Au03Driver::readStatus() {

	/*
	 * Read the status card
	 */
	int16 value;
	unsigned long not_needed;
	//	plExpCheckStatus(m_Handle, &value, &not_needed);

	// Status command
	owner()->computeAcquisitionStatus(0);
	owner()->computeDetectorStatus(0);

	if (owner()->startActivated() == true) {
		owner()->startActivated = false;
		synchroniseRead();
		owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	}
	if (owner()->m_SynchroniseReadActivated == true) {
		if (owner()->m_SynchroniseReadTerminated == true) {
			owner()->m_SynchroniseReadActivated = false;
			owner()->m_SynchroniseReadTerminated = false;
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		}
	} else {
		owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	}
}

/*
 * readInfos
 */
void RealDfk31Au03Driver::readInfos() {
}

}
