/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealStereoFluxgateDriver.h"
#include "StereoFluxgateDriver.h"
#include "StereoFluxgateDef.h"
#include "JtagMagnetometer.h"
#include "nEDM_MagChainBuilder.h"
#include <iostream>



using namespace std;

namespace stereo_fluxgate {

/*
 * Constructor
 */
RealStereoFluxgateDriver::RealStereoFluxgateDriver(StereoFluxgateDriver* owner) :
		StereoFluxgateState(owner) {


}

/*
 * Destructor
 */
RealStereoFluxgateDriver::~RealStereoFluxgateDriver() {


}


/*
 * init
 */
void RealStereoFluxgateDriver::init() {
	char *MicroControllerFile="/users/stereo/magnetometry/stereo_micro/micro.hex";
	char *FPGAFile="/users/stereo/magnetometry/fpga_controller.mcs";

	if (!Mag_API.Connect_USB(MicroControllerFile)) {
		cout << "Error : Fluxgate driver not connected" << endl;
		owner()->isconnected=false;
		return;
	}else{
		cout << "Fluxgate driver connected" << endl;
		owner()->isconnected=true;
	}
	if (!Mag_API.ConfigureFPGA(FPGAFile)) {
		cout << "Error : Fluxgate driver not configured" << endl;
		owner()->isconfigured=false;
		return;
	}else{
		cout << "Fluxgate connected" << endl;
		owner()->isconfigured=true;
	}

	if (!Mag_API.GoToIDLE())
		{
		 cout << "Connection to JTAG controller Go to IDLE failed" << endl;
		 owner()->isconfigured=false;
		 return;
		}

	if (!Mag_API.CheckIDCODEs())
		{
		cout << "Connection to JTAG devices , IDCODE check failed" << endl;
		owner()->isconfigured=false;
		return;
		}
	Mag_API.SetADCMode(ExtIn50Hz60HzRejectACal);
	Mag_API.DoConvert();
}

/*
 * read
 */
void RealStereoFluxgateDriver::readFieldValues() {
	double ValuesV[3*MagNumber];
	double ValuesB[3*MagNumber];
	int BinValues[3*MagNumber];
	bool OutOfRange[3*MagNumber];

	Mag_API.DoConvert();
	bool ret=Mag_API.GetADCValuesBin(BinValues,OutOfRange);
	if(!ret){
		cout << "Mag_API.GetADCValuesBin return false!!" << endl;
		return;
	}

	Mag_API.ConvertADCValuesV(BinValues,ValuesV);
	Mag_API.ConvertVValuesB(ValuesV,ValuesB);
	bool gOutOfRange=false;
	for(int ii=0;ii<3*MagNumber; ii++){
		if(OutOfRange[ii]) {
			cout << "Mag_API.GetADCValuesBin OutOfRange[" << ii <<"] return true!! binvalue="<< BinValues[ii] << " valuesV=" <<ValuesV[ii]<< " ValuesB="<<ValuesB[ii] << endl;
			gOutOfRange=true;
		}
	}
	for(int i=0;i<owner()->nbfluxgate();i++){
		owner()->Bx.set(i, ValuesB[3*i]);
		owner()->By.set(i, ValuesB[3*i+1]);
		owner()->Bz.set(i, ValuesB[3*i+2]);
	}

}

}
